@@ -1,14 +1,22 @@
-from typing import List, Optional, Union, overload, Type
+from __future__ import annotations
+
 import logging
+import threading
+from typing import overload, TYPE_CHECKING
+
 import archinfo
 from archinfo.arch_soot import ArchSoot, SootAddressDescriptor
 
+from .knowledge_plugins.functions import Function
 from .sim_state import SimState
 from .calling_conventions import default_cc, SimRegArg, SimStackArg, PointerWrapper, SimCCUnknown
 from .callable import Callable
-from .errors import AngrAssemblyError
-from .engines import UberEngine, ProcedureEngine, SimEngineConcrete, SimEngine
+from .errors import AngrError
+from .engines import UberEngine, ProcedureEngine
 from .sim_type import SimTypeFunction, SimTypeInt
+from .codenode import HookNode, SyscallNode
+from .block import Block, SootBlock
+from .sim_manager import SimulationManager
 
 try:
     from .engines import UberEnginePcode
@@ -16,6 +24,10 @@ try:
 except ImportError:
     UberEnginePcode = None
 
+if TYPE_CHECKING:
+    from angr import Project, SimCC
+    from angr.engines import SimEngine
+
 
 l = logging.getLogger(name=__name__)
 
@@ -25,15 +37,25 @@ class AngrObjectFactory:
     This factory provides access to important analysis elements.
     """
 
-    def __init__(self, project, default_engine: Optional[Type[SimEngine]] = None):
+    project: Project
+    default_engine_factory: type[SimEngine]
+    procedure_engine: ProcedureEngine
+    _default_cc: type[SimCC] | None
+
+    # We use thread local storage to cache engines on a per-thread basis
+    _tls: threading.local
+
+    def __init__(self, project, default_engine: type[SimEngine] | None = None):
+        self._tls = threading.local()
+
         if default_engine is None:
             if isinstance(project.arch, archinfo.ArchPcode) and UberEnginePcode is not None:
                 l.warning("Creating project with the experimental 'UberEnginePcode' engine")
-                default_engine_n = UberEnginePcode
+                self.default_engine_factory = UberEnginePcode
             else:
-                default_engine_n = UberEngine
+                self.default_engine_factory = UberEngine
         else:
-            default_engine_n = default_engine
+            self.default_engine_factory = default_engine
 
         if isinstance(project.arch, archinfo.ArchPcode):
             register_pcode_arch_default_cc(project.arch)
@@ -42,25 +64,31 @@ class AngrObjectFactory:
         self._default_cc = default_cc(
             project.arch.name, platform=project.simos.name if project.simos is not None else None, default=SimCCUnknown
         )
-        self.default_engine = default_engine_n(project)
         self.procedure_engine = ProcedureEngine(project)
 
-        if project.concrete_target:
-            self.concrete_engine = SimEngineConcrete(project)
-        else:
-            self.concrete_engine = None
+    def __getstate__(self):
+        return self.project, self.default_engine_factory, self.procedure_engine, self._default_cc
+
+    def __setstate__(self, state):
+        self.project, self.default_engine_factory, self.procedure_engine, self._default_cc = state
+        self._tls = threading.local()
+
+    @property
+    def default_engine(self):
+        if not hasattr(self._tls, "default_engine"):
+            self._tls.default_engine = self.default_engine_factory(self.project)
+        return self._tls.default_engine
 
     def snippet(self, addr, jumpkind=None, **block_opts):
         if self.project.is_hooked(addr) and jumpkind != "Ijk_NoHook":
             hook = self.project._sim_procedures[addr]
             size = hook.kwargs.get("length", 0)
             return HookNode(addr, size, self.project.hooked_by(addr))
-        elif self.project.simos.is_syscall_addr(addr):
+        if self.project.simos.is_syscall_addr(addr):
             syscall = self.project.simos.syscall_from_addr(addr)
             size = syscall.kwargs.get("length", 0)
             return SyscallNode(addr, size, syscall)
-        else:
-            return self.block(addr, **block_opts).codenode  # pylint: disable=no-member
+        return self.block(addr, **block_opts).codenode  # pylint: disable=no-member
 
     def successors(self, *args, engine=None, **kwargs):
         """
@@ -166,14 +194,12 @@ class AngrObjectFactory:
 
         grow_like_stack controls the behavior of allocating data at alloc_base. When data from args needs to be wrapped
         in a pointer, the pointer needs to point somewhere, so that data is dumped into memory at alloc_base. If you
-        set alloc_base to point to somewhere other than the stack, set grow_like_stack to False so that sequencial
+        set alloc_base to point to somewhere other than the stack, set grow_like_stack to False so that sequential
         allocations happen at increasing addresses.
         """
         return self.project.simos.state_call(addr, *args, **kwargs)
 
-    def simulation_manager(
-        self, thing: Optional[Union[List[SimState], SimState]] = None, **kwargs
-    ) -> "SimulationManager":
+    def simulation_manager(self, thing: list[SimState] | SimState | None = None, **kwargs) -> SimulationManager:
         """
         Constructs a new simulation manager.
 
@@ -199,7 +225,7 @@ class AngrObjectFactory:
         elif isinstance(thing, SimState):
             thing = [thing]
         else:
-            raise AngrError("BadType to initialze SimulationManager: %s" % repr(thing))
+            raise AngrError(f"BadType to initialize SimulationManager: {thing!r}")
 
         return SimulationManager(self.project, active_states=thing, **kwargs)
 
@@ -211,7 +237,7 @@ class AngrObjectFactory:
 
     def callable(
         self,
-        addr,
+        addr: int | Function,
         prototype=None,
         concrete_only=False,
         perform_merge=True,
@@ -220,22 +246,28 @@ class AngrObjectFactory:
         cc=None,
         add_options=None,
         remove_options=None,
+        step_limit: int | None = None,
     ):
         """
         A Callable is a representation of a function in the binary that can be interacted with like a native python
         function.
 
-        :param addr:            The address of the function to use
-        :param prototype:         The prototype of the call to use, as a string or a SimTypeFunction
+        :param addr:            The address of the function to use. If you pass in the function object, we will take
+                                its addr.
+        :param prototype:       The prototype of the call to use, as a string or a SimTypeFunction
         :param concrete_only:   Throw an exception if the execution splits into multiple states
         :param perform_merge:   Merge all result states into one at the end (only relevant if concrete_only=False)
         :param base_state:      The state from which to do these runs
         :param toc:             The address of the table of contents for ppc64
         :param cc:              The SimCC to use for a calling convention
+        :param step_limit:      The maximum number of blocks that Callable will execute before pruning the path.
         :returns:               A Callable object that can be used as a interface for executing guest code like a
                                 python function.
         :rtype:                 angr.callable.Callable
         """
+        if isinstance(addr, Function):
+            addr = addr.addr
+
         return Callable(
             self.project,
             addr=addr,
@@ -247,6 +279,7 @@ class AngrObjectFactory:
             cc=cc,
             add_options=add_options,
             remove_options=remove_options,
+            step_limit=step_limit,
         )
 
     def cc(self):
@@ -265,7 +298,7 @@ class AngrObjectFactory:
         """
         Return a default function prototype parameterized for this project and SimOS.
         """
-        return SimTypeFunction(tuple(), SimTypeInt()).with_arch(self.project.arch)
+        return SimTypeFunction((), SimTypeInt()).with_arch(self.project.arch)
 
     # pylint: disable=unused-argument, no-self-use, function-redefined
     @overload
@@ -275,7 +308,6 @@ class AngrObjectFactory:
         size=None,
         max_size=None,
         byte_string=None,
-        vex=None,
         thumb=False,
         backup_state=None,
         extra_stop_points=None,
@@ -283,13 +315,14 @@ class AngrObjectFactory:
         num_inst=None,
         traceflags=0,
         insn_bytes=None,
-        insn_text=None,  # backward compatibility
         strict_block_end=None,
         collect_data_refs=False,
         cross_insn_opt=True,
         load_from_ro_regions=False,
+        const_prop=False,
         initial_regs=None,
-    ) -> "Block": ...
+        skip_stmts=False,
+    ) -> Block: ...
 
     # pylint: disable=unused-argument, no-self-use, function-redefined
     @overload
@@ -299,7 +332,6 @@ class AngrObjectFactory:
         size=None,
         max_size=None,
         byte_string=None,
-        vex=None,
         thumb=False,
         backup_state=None,
         extra_stop_points=None,
@@ -307,11 +339,13 @@ class AngrObjectFactory:
         num_inst=None,
         traceflags=0,
         insn_bytes=None,
-        insn_text=None,  # backward compatibility
         strict_block_end=None,
         collect_data_refs=False,
+        load_from_ro_regions=False,
+        const_prop=False,
         cross_insn_opt=True,
-    ) -> "SootBlock": ...
+        skip_stmts=False,
+    ) -> SootBlock: ...
 
     def block(
         self,
@@ -319,7 +353,6 @@ class AngrObjectFactory:
         size=None,
         max_size=None,
         byte_string=None,
-        vex=None,
         thumb=False,
         backup_state=None,
         extra_stop_points=None,
@@ -327,39 +360,26 @@ class AngrObjectFactory:
         num_inst=None,
         traceflags=0,
         insn_bytes=None,
-        insn_text=None,  # backward compatibility
         strict_block_end=None,
         collect_data_refs=False,
         cross_insn_opt=True,
         load_from_ro_regions=False,
+        const_prop=False,
         initial_regs=None,
+        skip_stmts=False,
     ):
         if isinstance(self.project.arch, ArchSoot) and isinstance(addr, SootAddressDescriptor):
             return SootBlock(addr, arch=self.project.arch, project=self.project)
 
-        if insn_bytes is not None and insn_text is not None:
-            raise AngrError("You cannot provide both 'insn_bytes' and 'insn_text'!")
-
         if insn_bytes is not None:
             byte_string = insn_bytes
 
-        if insn_text is not None:
-            byte_string = self.project.arch.asm(insn_text, addr=addr, as_bytes=True, thumb=thumb)
-            if byte_string is None:
-                # assembly failed
-                raise AngrAssemblyError(
-                    "Assembling failed. Please make sure keystone is installed, and the assembly string is correct."
-                )
-
-        if max_size is not None:
-            l.warning('Keyword argument "max_size" has been deprecated for block(). Please use "size" instead.')
-            size = max_size
         return Block(
             addr,
             project=self.project,
             size=size,
+            max_size=max_size,
             byte_string=byte_string,
-            vex=vex,
             extra_stop_points=extra_stop_points,
             thumb=thumb,
             backup_state=backup_state,
@@ -370,7 +390,9 @@ class AngrObjectFactory:
             collect_data_refs=collect_data_refs,
             cross_insn_opt=cross_insn_opt,
             load_from_ro_regions=load_from_ro_regions,
+            const_prop=const_prop,
             initial_regs=initial_regs,
+            skip_stmts=skip_stmts,
         )
 
     def fresh_block(self, addr, size, backup_state=None):
@@ -380,9 +402,3 @@ class AngrObjectFactory:
     cc.SimStackArg = SimStackArg
     callable.PointerWrapper = PointerWrapper
     call_state.PointerWrapper = PointerWrapper
-
-
-from .errors import AngrError
-from .sim_manager import SimulationManager
-from .codenode import HookNode, SyscallNode
-from .block import Block, SootBlock
