@@ -1,16 +1,19 @@
 # pylint:disable=abstract-method,arguments-differ,assignment-from-no-return
+from __future__ import annotations
 import logging
-from typing import Optional, List, Set, Tuple, Union, Callable, Any, FrozenSet
+from typing import Any
+from collections.abc import Callable
 
+from angr.storage.memory_mixins.memory_mixin import MemoryMixin
 from angr.utils.dynamic_dictlist import DynamicDictList
-from .....storage.memory_object import SimMemoryObject, SimLabeledMemoryObject
-from . import PageBase
+from angr.storage.memory_object import SimMemoryObject, SimLabeledMemoryObject
+from .base import PageBase
 from .cooperation import MemoryObjectSetMixin
 
 
 l = logging.getLogger(name=__name__)
 
-_MOTYPE = Union[SimMemoryObject, SimLabeledMemoryObject]
+_MOTYPE = SimMemoryObject | SimLabeledMemoryObject
 
 
 class MVListPage(
@@ -18,31 +21,27 @@ class MVListPage(
     PageBase,
 ):
     """
-    MVListPage allows storing multiple values at the same location, thus allowing weak updates.
+    MVListPage allows storing multiple values at the same location.
 
-    Each store() may take a value or multiple values, and a "weak" parameter to specify if this store is a weak update
-    or not.
+    Each store() may take a value or multiple values.
     Each load() returns an iterator of all values stored at that location.
     """
 
     def __init__(self, memory=None, content=None, sinkhole=None, mo_cmp=None, **kwargs):
         super().__init__(**kwargs)
 
-        self.content: DynamicDictList[Optional[Union[_MOTYPE, Set[_MOTYPE]]]] = (
+        self.content: DynamicDictList[_MOTYPE | set[_MOTYPE] | None] = (
             DynamicDictList(max_size=memory.page_size, content=content) if content is not None else None
         )
         self.stored_offset = set()
-        self._mo_cmp: Optional[Callable] = mo_cmp
+        self._mo_cmp: Callable | None = mo_cmp
 
-        if self.content is None:
-            if memory is not None:
-                self.content: DynamicDictList[Optional[Union[_MOTYPE, Set[_MOTYPE]]]] = DynamicDictList(
-                    max_size=memory.page_size
-                )
+        if self.content is None and memory is not None:
+            self.content: DynamicDictList[_MOTYPE | set[_MOTYPE] | None] = DynamicDictList(max_size=memory.page_size)
 
-        self.sinkhole: Optional[_MOTYPE] = sinkhole
+        self.sinkhole: _MOTYPE | None = sinkhole
 
-    def copy(self, memo) -> "MVListPage":
+    def copy(self, memo) -> MVListPage:
         o = super().copy(memo)
         o.content = DynamicDictList(max_size=self.content.max_size, content=self.content)
         o.sinkhole = self.sinkhole
@@ -52,7 +51,7 @@ class MVListPage(
 
     def load(
         self, addr, size=None, endness=None, page_addr=None, memory=None, cooperate=False, **kwargs
-    ) -> List[Tuple[int, _MOTYPE]]:
+    ) -> list[tuple[int, _MOTYPE]]:
         result = []
         last_seen = ...  # ;)
 
@@ -105,35 +104,25 @@ class MVListPage(
             self.stored_offset.add(subaddr)
         result[-1] = (global_start_addr, new_item)
 
-    def store(self, addr, data, size=None, endness=None, memory=None, cooperate=False, weak=False, **kwargs):
-        super().store(addr, data, size=size, endness=endness, memory=memory, cooperate=cooperate, weak=weak, **kwargs)
+    def store(self, addr, data, size=None, endness=None, memory=None, cooperate=False, **kwargs):
+        super().store(addr, data, size=size, endness=endness, memory=memory, cooperate=cooperate, **kwargs)
 
         if not cooperate:
             data = self._force_store_cooperation(addr, data, size, endness, memory=memory, **kwargs)
 
-        data: Set[_MOTYPE]
+        data: set[_MOTYPE]
 
         if size == len(self.content) and addr == 0 and len(data) == 1:
             self.sinkhole = next(iter(data))
             self.content = DynamicDictList(max_size=len(self.content))
             self.stored_offset = set()
         else:
-            if not weak:
-                if len(data) == 1:
-                    # unpack
-                    data: _MOTYPE = next(iter(data))
-                for subaddr in range(addr, addr + size):
-                    self.content[subaddr] = data
-                    self.stored_offset.add(subaddr)
-            else:
-                for subaddr in range(addr, addr + size):
-                    if self.content[subaddr] is None:
-                        self.content[subaddr] = data
-                    elif type(self.content[subaddr]) is set:
-                        self.content[subaddr] |= data
-                    else:
-                        self.content[subaddr] = {self.content[subaddr]} | data
-                    self.stored_offset.add(subaddr)
+            if len(data) == 1:
+                # unpack
+                data: _MOTYPE = next(iter(data))
+            for subaddr in range(addr, addr + size):
+                self.content[subaddr] = data
+                self.stored_offset.add(subaddr)
 
     def erase(self, addr, size=None, **kwargs) -> None:
         for off in range(size):
@@ -141,19 +130,20 @@ class MVListPage(
 
     def merge(
         self,
-        others: List["MVListPage"],
+        others: list[MVListPage],
         merge_conditions,
         common_ancestor=None,
-        page_addr: int = None,
-        memory=None,
-        changed_offsets: Optional[Set[int]] = None,
+        *,
+        page_addr: int,
+        memory: MemoryMixin,
+        changed_offsets: set[int] | None = None,
     ):
         if changed_offsets is None:
             changed_offsets = set()
             for other in others:
                 changed_offsets |= self.changed_bytes(other, page_addr)
 
-        all_pages: List["MVListPage"] = [self] + others
+        all_pages: list[MVListPage] = [self, *others]
         if merge_conditions is None:
             merge_conditions = [None] * len(all_pages)
 
@@ -166,7 +156,7 @@ class MVListPage(
                 continue
             l.debug("... on byte 0x%x", b)
 
-            memory_object_sets: Set[Tuple[FrozenSet[SimMemoryObject], Any]] = set()
+            memory_object_sets: set[tuple[frozenset[SimMemoryObject], Any]] = set()
             unconstrained_in = []
 
             # first get a list of all memory objects at that location, and all memories that don't have those bytes
@@ -216,7 +206,7 @@ class MVListPage(
                         to_merge.append((mo.object, fv))
 
                 # Update `merged_to`
-                mo_base = list(mo_bases)[0]
+                mo_base = next(iter(mo_bases))
                 mo_length = next(iter(mo_lengths))
                 size = min(mo_length - (page_addr + b - mo_base), len(self.content) - b)
                 merged_to = b + size
@@ -231,7 +221,7 @@ class MVListPage(
                 # new_object = self._replace_memory_object(our_mo, merged_val, page_addr, memory.page_size)
 
                 new_mos = {SimMemoryObject(v, mo_base, endness=the_endness) for v in merged_val}
-                self.store(b, new_mos, size=size, cooperate=True, weak=False)
+                self.store(b, new_mos, size=size, cooperate=True)
 
                 merged_offsets.add(b)
 
@@ -244,7 +234,7 @@ class MVListPage(
                     for mo in mo_set:
                         min_size = min(min_size, mo.length - ((b + page_addr - mo.base) & mask))
                 for um, _ in unconstrained_in:
-                    for i in range(0, min_size):
+                    for i in range(min_size):
                         if um._contains(b + i, page_addr):
                             min_size = i
                             break
@@ -272,14 +262,14 @@ class MVListPage(
                     continue
 
                 new_mos = {SimMemoryObject(v, page_addr + b, endness="Iend_BE") for v in merged_val}
-                self.store(b, new_mos, size=min_size, cooperate=True, weak=False)
+                self.store(b, new_mos, size=min_size, cooperate=True)
                 merged_offsets.add(b)
 
         self.stored_offset |= merged_offsets
         return merged_offsets
 
     def compare(
-        self, other: "MVListPage", page_addr: int = None, memory=None, changed_offsets=None
+        self, other: MVListPage, page_addr: int | None = None, memory=None, changed_offsets=None
     ) -> bool:  # pylint: disable=unused-argument
         compared_to = None
         for b in sorted(changed_offsets):
@@ -288,7 +278,7 @@ class MVListPage(
 
             unconstrained_in = []
             self_has_memory_object_set = False
-            memory_object_sets: Set[FrozenSet[SimMemoryObject]] = set()
+            memory_object_sets: set[frozenset[SimMemoryObject]] = set()
             for sm in [self, other]:
                 if sm._contains(b, page_addr):
                     memory_objects = set()
@@ -311,15 +301,15 @@ class MVListPage(
 
         return True
 
-    def changed_bytes(self, other: "MVListPage", page_addr: int = None):
-        candidates: Set[int] = super().changed_bytes(other)
+    def changed_bytes(self, other: MVListPage, page_addr: int | None = None):
+        candidates: set[int] = super().changed_bytes(other)
         if candidates is not None:
             # using the result from the history tracking mixin as an approximation
             return candidates
 
         # slower path
         if candidates is None:
-            candidates: Set[int] = set()
+            candidates: set[int] = set()
             # resort to the slower solution
             if self.sinkhole is None:
                 candidates |= self.stored_offset
@@ -336,31 +326,27 @@ class MVListPage(
                         candidates.add(i)
 
         byte_width = 8  # TODO: Introduce self.state if we want to use self.state.arch.byte_width
-        differences: Set[int] = set()
+        differences: set[int] = set()
         for c in candidates:
             s_contains = self._contains(c, page_addr)
             o_contains = other._contains(c, page_addr)
-            if not s_contains and o_contains:
-                differences.add(c)
-            elif s_contains and not o_contains:
+            if (not s_contains and o_contains) or (s_contains and not o_contains):
                 differences.add(c)
             else:
-                if self.content[c] is None:
-                    if self.sinkhole is not None:
-                        self.content[c] = SimMemoryObject(
-                            self.sinkhole.bytes_at(page_addr + c, 1),
-                            page_addr + c,
-                            byte_width=byte_width,
-                            endness="Iend_BE",
-                        )
-                if other.content[c] is None:
-                    if other.sinkhole is not None:
-                        other.content[c] = SimMemoryObject(
-                            other.sinkhole.bytes_at(page_addr + c, 1),
-                            page_addr + c,
-                            byte_width=byte_width,
-                            endness="Iend_BE",
-                        )
+                if self.content[c] is None and self.sinkhole is not None:
+                    self.content[c] = SimMemoryObject(
+                        self.sinkhole.bytes_at(page_addr + c, 1),
+                        page_addr + c,
+                        byte_width=byte_width,
+                        endness="Iend_BE",
+                    )
+                if other.content[c] is None and other.sinkhole is not None:
+                    other.content[c] = SimMemoryObject(
+                        other.sinkhole.bytes_at(page_addr + c, 1),
+                        page_addr + c,
+                        byte_width=byte_width,
+                        endness="Iend_BE",
+                    )
                 if s_contains and self.content[c] != other.content[c]:
                     same = None
                     if self._mo_cmp is not None:
@@ -410,14 +396,14 @@ class MVListPage(
         return new_mo
 
     @staticmethod
-    def _resolve_range(mo: SimMemoryObject, page_addr: int, page_size) -> Tuple[int, int]:
+    def _resolve_range(mo: SimMemoryObject, page_addr: int, page_size) -> tuple[int, int]:
         start = max(mo.base, page_addr)
         end = min(mo.last_addr + 1, page_addr + page_size)
         if end <= start:
             l.warning("Nothing left of the memory object to store in SimPage.")
         return start, end
 
-    def _get_objects(self, start: int, page_addr: int) -> Optional[List[SimMemoryObject]]:
+    def _get_objects(self, start: int, page_addr: int) -> list[SimMemoryObject] | None:
         mos = self.content[start]
         if mos is None:
             return None
